"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const bin_wrapper_progress_1 = require("@akashaproject/bin-wrapper-progress");
const path = require("path");
const Promise = require("bluebird");
const fs_1 = require("fs");
const defaultTarget = path.join(__dirname, 'bin');
const repo = 'https://gethstore.blob.core.windows.net/builds/';
const gethVersion = '1.8.27';
const gethCommit = '4bcc0a37';
const source = {
    linux: `${repo}geth-linux-amd64-${gethVersion}-${gethCommit}.tar.gz`,
    win: `${repo}geth-windows-amd64-${gethVersion}-${gethCommit}.zip`,
    osx: `${repo}geth-darwin-amd64-${gethVersion}-${gethCommit}.tar.gz`,
};
class GethBin {
    constructor(target = defaultTarget) {
        this.wrapper = new bin_wrapper_progress_1.default()
            .src(source.linux, 'linux', 'x64')
            .src(source.win, 'win32', 'x64')
            .src(source.osx, 'darwin', 'x64')
            .dest(target)
            .use(process.platform === 'win32' ? 'geth.exe' : 'geth');
    }
    static requiredVersion() {
        return `v${gethVersion}`;
    }
    getPath() {
        return this.wrapper.path();
    }
    check(cb) {
        this.wrapper.run(['version'], (err) => {
            if (err) {
                return cb(err);
            }
            const response = { binPath: this.getPath() };
            cb('', response);
        });
    }
    deleteBin() {
        const path = this.getPath();
        return Promise.fromCallback((cb) => fs_1.unlink(path, cb));
    }
}
exports.GethBin = GethBin;
